@@ -1,5 +1,5 @@
 /*  xmms-shn - a shorten (.shn) plugin for XMMS
- *  Copyright (C) 2000-2004  Jason Jordan <shnutils@freeshell.org>
+ *  Copyright (C) 2000-2007  Jason Jordan <shnutils@freeshell.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  */
 
 /*
- * $Id: shn.h,v 1.25 2004/05/04 02:26:36 jason Exp $
+ * $Id: shn.h,v 1.27 2007/03/23 05:49:48 jason Exp $
  */
 
 #ifndef _SHN_H
@@ -254,7 +254,10 @@ typedef struct _shn_vars
 	ulong  last_file_position_no_really;
 	ulong  initial_file_position;
 	ulong  bytes_read;
-	unsigned short bitshift;
+	unsigned short actual_bitshift;
+	int    actual_maxnlpc;
+	int    actual_nmean;
+	int    actual_nchan;
 	long   seek_offset;
 } shn_vars;
 
