@@ -1,5 +1,5 @@
 /*  shn.c - main functions for xmms-shn
- *  Copyright (C) 2000-2004  Jason Jordan <shnutils@freeshell.org>
+ *  Copyright (C) 2000-2007  Jason Jordan <shnutils@freeshell.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  */
 
 /*
- * $Id: shn.c,v 1.35 2004/05/04 02:26:36 jason Exp $
+ * $Id: shn.c,v 1.38 2007/03/23 05:49:48 jason Exp $
  */
 
 #include <stdio.h>
@@ -278,6 +278,7 @@ int get_wave_header(shn_file *this_shn)
     }
 
     nchan = UINT_GET(CHANSIZE, this_shn);
+    this_shn->vars.actual_nchan = nchan;
 
     /* get blocksize if version > 0 */
     if(version > 0)
@@ -285,11 +286,13 @@ int get_wave_header(shn_file *this_shn)
       int byte;
       blocksize = UINT_GET((int) (log((double) DEFAULT_BLOCK_SIZE) / M_LN2),this_shn);
       maxnlpc = UINT_GET(LPCQSIZE, this_shn);
+      this_shn->vars.actual_maxnlpc = maxnlpc;
       nmean = UINT_GET(0, this_shn);
+      this_shn->vars.actual_nmean = nmean;
       nskip = UINT_GET(NSKIPSIZE, this_shn);
       for(i = 0; i < nskip; i++)
       {
-	byte = uvar_get(XBYTESIZE,this_shn);
+        byte = uvar_get(XBYTESIZE,this_shn);
       }
     }
     else
@@ -517,7 +520,7 @@ int get_wave_header(shn_file *this_shn)
 
           case FN_BITSHIFT:
             bitshift = uvar_get(BITSHIFTSIZE,this_shn);
-            this_shn->vars.bitshift = bitshift;
+            this_shn->vars.actual_bitshift = bitshift;
             break;
 
           default:
@@ -617,7 +620,10 @@ shn_file *load_shn(char *filename)
 	tmp_file->vars.last_file_position = 0;
 	tmp_file->vars.last_file_position_no_really = 0;
 	tmp_file->vars.bytes_read = 0;
-	tmp_file->vars.bitshift = 0;
+	tmp_file->vars.actual_bitshift = 0;
+	tmp_file->vars.actual_maxnlpc = 0;
+	tmp_file->vars.actual_nmean = 0;
+	tmp_file->vars.actual_nchan = 0;
 	tmp_file->vars.seek_offset = 0;
 
 	tmp_file->decode_state = NULL;
@@ -684,25 +690,44 @@ shn_file *load_shn(char *filename)
 
 	if (NO_SEEK_TABLE != tmp_file->vars.seek_table_entries)
 	{
+		/* verify seek tables */
+
 		first_seek_table = (shn_seek_entry *)tmp_file->seek_table;
 
-		/* check for broken seek tables - if found, disable seeking */
-		if (0 == tmp_file->seek_header.version)
+		if (tmp_file->vars.actual_bitshift != shn_uchar_to_ushort_le(first_seek_table->data+22))
 		{
-			/* test, if the bitshift value in the file is identical to the bitshift value of the first seektable entry */
-			if (tmp_file->vars.bitshift != shn_uchar_to_ushort_le(first_seek_table->data+22))
-			{
-				shn_debug("Broken seek table detected - seeking disabled for file '%s'.",tmp_file->wave_header.filename);
-				tmp_file->vars.seek_table_entries = NO_SEEK_TABLE;
-			}
+			/* initial bitshift value in the file does not match the first bitshift value of the first seektable entry - seeking is broken */
+			shn_debug("Broken seek table detected (invalid bitshift) - seeking disabled for this file.");
+			tmp_file->vars.seek_table_entries = NO_SEEK_TABLE;
 		}
-
-		tmp_file->vars.seek_offset += tmp_file->vars.initial_file_position - shn_uchar_to_ulong_le(first_seek_table->data+8);
-
-		if (0 != tmp_file->vars.seek_offset)
+		else if (tmp_file->vars.actual_nchan > 2)
 		{
-			shn_debug("Adjusting seek table offsets by %ld bytes due to mismatch between seek table values and input file - seeking might not work correctly.",
-				tmp_file->vars.seek_offset);
+			/* nchan is greater than the number of such entries stored in a seek table entry - seeking won't work */
+			shn_debug("Broken seek table detected (nchan %d not in range [1 .. 2]) - seeking disabled for this file.",tmp_file->vars.actual_nchan);
+			tmp_file->vars.seek_table_entries = NO_SEEK_TABLE;
+		}
+		else if (tmp_file->vars.actual_maxnlpc > 3)
+		{
+			/* maxnlpc is greater than the number of such entries stored in a seek table entry - seeking won't work */
+			shn_debug("Broken seek table detected (maxnlpc %d not in range [0 .. 3]) - seeking disabled for this file.",tmp_file->vars.actual_maxnlpc);
+			tmp_file->vars.seek_table_entries = NO_SEEK_TABLE;
+		}
+		else if (tmp_file->vars.actual_nmean > 4)
+		{
+			/* nmean is greater than the number of such entries stored in a seek table entry - seeking won't work */
+			shn_debug("Broken seek table detected (nmean %d not in range [0 .. 4]) - seeking disabled for this file.",tmp_file->vars.actual_nmean);
+			tmp_file->vars.seek_table_entries = NO_SEEK_TABLE;
+		}
+		else
+		{
+			/* seek table appears to be valid - now adjust byte offsets in seek table to match the file */
+			tmp_file->vars.seek_offset += tmp_file->vars.initial_file_position - shn_uchar_to_ulong_le(first_seek_table->data+8);
+
+			if (0 != tmp_file->vars.seek_offset)
+			{
+				shn_debug("Adjusting seek table offsets by %ld bytes due to mismatch between seek table values and input file - seeking might not work correctly.",
+					tmp_file->vars.seek_offset);
+			}
 		}
 	}
 
@@ -822,8 +847,8 @@ restart:
     if(FORMAT_VERSION < 2)
     {
       for(i = 0; i < strlen(magic); i++)
-	if(getc_exit(this_shn->vars.fd) != magic[i]) {
-	  shn_error_fatal(this_shn,"Bad magic number");
+        if(getc_exit(this_shn->vars.fd) != magic[i]) {
+          shn_error_fatal(this_shn,"Bad magic number");
           goto exit_thread;
         }
 
@@ -905,7 +930,7 @@ restart:
       nskip = UINT_GET(NSKIPSIZE, this_shn);
       for(i = 0; i < nskip; i++)
       {
-	byte = uvar_get(XBYTESIZE,this_shn);
+        byte = uvar_get(XBYTESIZE,this_shn);
       }
     }
     else
@@ -1093,28 +1118,22 @@ restart:
               if (this_shn->vars.seek_to != -1)
               {
                 shn_seek_entry *seek_info;
+                int j;
 
                 shn_debug("Seeking to %d:%02d",this_shn->vars.seek_to/60,this_shn->vars.seek_to%60);
 
                 seek_info = shn_seek_entry_search(this_shn->seek_table,this_shn->vars.seek_to * (ulong)this_shn->wave_header.samples_per_sec,0,
 							      (ulong)(this_shn->vars.seek_table_entries - 1),this_shn->vars.seek_resolution);
 
-                buffer[0][-1] = shn_uchar_to_slong_le(seek_info->data+24);
-                buffer[0][-2] = shn_uchar_to_slong_le(seek_info->data+28);
-                buffer[0][-3] = shn_uchar_to_slong_le(seek_info->data+32);
-                offset[0][0]  = shn_uchar_to_slong_le(seek_info->data+48);
-                offset[0][1]  = shn_uchar_to_slong_le(seek_info->data+52);
-                offset[0][2]  = shn_uchar_to_slong_le(seek_info->data+56);
-                offset[0][3]  = shn_uchar_to_slong_le(seek_info->data+60);
+                /* loop through number of channels in this file */
+                for (i=0;i<nchan;i++) {
+                  /* load the three sample buffer values for this channel */
+                  for (j=0;j<3;j++)
+                    buffer[i][j-3] = shn_uchar_to_slong_le(seek_info->data+32+12*i-4*j);
 
-                if (nchan > 1) {
-                  buffer[1][-1] = shn_uchar_to_slong_le(seek_info->data+36);
-                  buffer[1][-2] = shn_uchar_to_slong_le(seek_info->data+40);
-                  buffer[1][-3] = shn_uchar_to_slong_le(seek_info->data+44);
-                  offset[1][0]  = shn_uchar_to_slong_le(seek_info->data+64);
-                  offset[1][1]  = shn_uchar_to_slong_le(seek_info->data+68);
-                  offset[1][2]  = shn_uchar_to_slong_le(seek_info->data+72);
-                  offset[1][3]  = shn_uchar_to_slong_le(seek_info->data+76);
+                  /* load the variable number of offset history values for this channel */
+                  for (j=0;j<MAX(1,nmean);j++)
+                    offset[i][j]  = shn_uchar_to_slong_le(seek_info->data+48+16*i+4*j);
                 }
 
                 bitshift = shn_uchar_to_ushort_le(seek_info->data+22);
@@ -1162,7 +1181,7 @@ restart:
                 if(maxnlpc > 0 && qlpc)
                   free((void *) qlpc);
 
-		fseek(this_shn->vars.fd,0,SEEK_SET);
+                fseek(this_shn->vars.fd,0,SEEK_SET);
                 goto restart;
               }
               else
